###########################################################################
#                                                                         #
#            Routines for new edit graph motif                            #
#                                                                         #
###########################################################################

proc powEditGraphDlg { gn } {
    global powbg currgn
    global powEditGraphname powEditDisplayOpt

    set powEditGraphname $gn

    if {[winfo exists .powedit]} {destroy .powedit}
    powToplevel .powedit .pow "-bg $powbg"
    catch {wm title .powedit "Edit Graph"}

    label .powedit.graphCont -text "Graph Contents:" -fg black -bg yellow
    frame .powedit.graphList -bg $powbg
    scrollbar .powedit.graphList.scrolly -orient vertical -takefocus 0\
	    -command {.powedit.graphList.lst yview} -bg $powbg
    listbox .powedit.graphList.lst -bg $powbg -width 25 -height 6 \
	    -selectmode browse -exportselection 0 -takefocus 0 \
	    -yscrollcommand {.powedit.graphList.scrolly set }
    bind .powedit.graphList.lst <ButtonRelease-1> \
	    { if { $powEditDisplayOpt!="graph" } powEditBuildOptions }

    grid .powedit.graphList.lst     -in .powedit.graphList -row 1 -column 1 \
	    -sticky news
    grid .powedit.graphList.scrolly -in .powedit.graphList -row 1 -column 2 \
	    -sticky news
    grid rowconfigure    .powedit.graphList 1 -weight 1
    grid columnconfigure .powedit.graphList 1 -weight 1

    label .powedit.elemCont -text "Available Objects:" -fg black -bg yellow
    frame .powedit.elemList -bg $powbg
    scrollbar .powedit.elemList.scrolly -orient vertical -takefocus 0 \
	    -command {.powedit.elemList.lst yview} -bg $powbg
    listbox .powedit.elemList.lst -bg $powbg -width 25 -height 6 \
	    -selectmode browse -exportselection 0 -takefocus 0 \
	    -yscrollcommand {.powedit.elemList.scrolly set }

    grid .powedit.elemList.lst     -in .powedit.elemList -row 1 -column 1 \
	    -sticky news
    grid .powedit.elemList.scrolly -in .powedit.elemList -row 1 -column 2 \
	    -sticky news
    grid rowconfigure    .powedit.elemList 1 -weight 1
    grid columnconfigure .powedit.elemList 1 -weight 1

    button .powedit.help -bg $powbg -text "Help" \
	    -command {powHelp EditGraphs.html} -takefocus 0

    grid .powedit.graphCont -in .powedit -row 1 -column 1 -sticky w \
	    -padx 5 -pady 5
    grid .powedit.graphList -in .powedit -row 2 -column 1 -sticky news
    grid .powedit.help      -in .powedit -row 1 -column 2
    grid .powedit.elemCont  -in .powedit -row 1 -column 3 -sticky w \
	    -padx 5 -pady 5
    grid .powedit.elemList  -in .powedit -row 2 -column 3 -sticky news

    frame  .powedit.elemButt -bg $powbg
    button .powedit.elemButt.add    -text "<-- Add" -bg $powbg -takefocus 0 \
	    -command {powEditAddElems}
    button .powedit.elemButt.remove -text "--> Remove" -bg $powbg -takefocus 0 \
	    -command {powEditRemoveElems}

    pack .powedit.elemButt.add    -in .powedit.elemButt -fill x \
	    -padx 5 -pady 2
    pack .powedit.elemButt.remove -in .powedit.elemButt -fill x \
	    -padx 5 -pady 2
    grid .powedit.elemButt        -in .powedit -row 2 -column 2

    frame .powedit.graphName -bg $powbg
    label .powedit.graphName.lab -bg yellow -fg black -text "Graph Name:"
    entry .powedit.graphName.ent -bg $powbg -textvariable powEditGraphname

    pack .powedit.graphName.lab -in .powedit.graphName -side left
    pack .powedit.graphName.ent -in .powedit.graphName -side left -padx 5
    grid .powedit.graphName     -in .powedit -row 3 -column 1 -columnspan 2 \
	    -sticky w -padx 18

    button .powedit.editObj -text "Edit Objects" -bg $powbg -takefocus 0 \
	    -command {powEditObjectDlg}
    grid .powedit.editObj -in .powedit -row 3 -column 3 -padx 5 -pady 5

    frame .powedit.displayOpt -bg $powbg -relief ridge -bd 3
    label .powedit.displayOpt.lab -bg yellow -fg black -text "Display Options for..."
    radiobutton .powedit.displayOpt.graph -bg $powbg -text Graph \
	    -variable powEditDisplayOpt -value graph \
	    -highlightthickness 0 -takefocus 0
    radiobutton .powedit.displayOpt.element -bg $powbg -text "Selected Object" \
	    -variable powEditDisplayOpt -value element \
	    -highlightthickness 0 -takefocus 0
    set powEditDisplayOpt graph

    pack  .powedit.displayOpt.lab -in .powedit.displayOpt -side left
    pack  .powedit.displayOpt.graph -in .powedit.displayOpt -side left -padx 10
    pack  .powedit.displayOpt.element -in .powedit.displayOpt -side left -padx 10
    grid  .powedit.displayOpt -in .powedit -row 4 -column 1 -sticky swe \
	    -columnspan 3 -padx 15

    frame .powedit.btns -bg $powbg
    button .powedit.btns.create -text "Create Graph" -bg $powbg \
	    -command {
	powEditUpdateVariables
	set gn $powEditGraphname
	powUnmapGraph $gn 1
	powCreateGraph $gn $powPlotParam(curves,$gn) $powPlotParam(images,$gn) \
		$powPlotParam(xunits,$gn) $powPlotParam(yunits,$gn) \
		$powPlotParam(xlabel,$gn) $powPlotParam(ylabel,$gn) \
		$powPlotParam(xdimdisp,$gn) $powPlotParam(ydimdisp,$gn) \
		$powPlotParam(xBot,$gn) $powPlotParam(yBot,$gn) \
		$powPlotParam(xTop,$gn) $powPlotParam(yTop,$gn)
    }
    button .powedit.btns.reload -text "Reload Info" -bg $powbg \
	    -command {
	set powEditGraphname $currgn
	powEditInitVariables
	powEditBuildOptions
    }
    button .powedit.btns.exit -text "Exit"  -bg $powbg \
	    -command {destroy .powedit}

    pack .powedit.btns.create -in .powedit.btns -side left \
	    -padx 5 -pady 2
    pack .powedit.btns.reload -in .powedit.btns -side left \
	    -padx 5 -pady 2
    pack .powedit.btns.exit   -in .powedit.btns -side left \
	    -padx 5 -pady 2
    grid .powedit.btns        -in .powedit -row 6 -column 1 -columnspan 3 \
	    -pady 5

    grid rowconfigure    .powedit 2 -weight 1
    grid rowconfigure    .powedit 5 -weight 1

    grid columnconfigure .powedit 1 -weight 1
    grid columnconfigure .powedit 3 -weight 1

    powEditInitVariables
    powEditBuildOptions
    trace variable powEditDisplayOpt w powChangeBuildOptions
}

proc powEditEmptyOptions { frame str } {
    global powbg

    if { [winfo exists $frame] } {
	destroy $frame
    }
    frame $frame -bg $powbg
    label $frame.lab -bg $powbg -text $str
    grid $frame.lab -in $frame
}

proc powChangeBuildOptions { a b c } {
    powEditBuildOptions
}

proc powEditBuildOptions { } {
    global powbg powEditDisplayOpt
    
    if { [winfo exists .powedit.displayOpt1] } {
	set oldframe .powedit.displayOpt1
	set newframe .powedit.displayOpt2
    } else {
	set oldframe .powedit.displayOpt2
	set newframe .powedit.displayOpt1
    }

    if { $powEditDisplayOpt=="graph" } {
	powEditGraphOptions $newframe
    } else {
	set itemNo [.powedit.graphList.lst curselection]
	if {$itemNo==""} {
	    powEditEmptyOptions $newframe "No element selected for graph"
	} else {
	    set item [.powedit.graphList.lst get $itemNo]
	    foreach {name type} [split $item { }] {}
	    if { $type == "(curve)" } {
		powEditCurveOptions $newframe $name
	    } else {
		powEditEmptyOptions $newframe "No options available for images"
	    }
	}
    }
    $newframe configure -borderwidth 3 -relief ridge
    grid $newframe -in .powedit -row 5 -column 1 -padx 15 -pady 5 \
	    -columnspan 3 -sticky news
    catch {destroy $oldframe}
}

proc powEditAddElems { } {
    global powEditGraphname powEditDisplayOpt
    global powEditPlotParam powEditCurveParam

    set elemNo  [.powedit.elemList.lst  curselection]
    if { $elemNo != "" } {

	set item [.powedit.elemList.lst get $elemNo]
	.powedit.elemList.lst delete $elemNo
	if { [.powedit.elemList.lst size]<=$elemNo } {
	    .powedit.elemList.lst selection set end
	} else {
	    .powedit.elemList.lst selection set $elemNo
	}
	.powedit.graphList.lst insert end $item
	.powedit.graphList.lst selection clear 0 end
	.powedit.graphList.lst selection set end
	foreach {name type} [split $item { }] {}
	if { $type=="(curve)" } {
	    if { $powEditPlotParam(curves,new)=="NULL" } {
		set powEditPlotParam(curves,new) $name
	    } else {
		lappend powEditPlotParam(curves,new) $name
	    }
	} else {
	    if { $powEditPlotParam(images,new)=="NULL" } {
		set powEditPlotParam(images,new) $name
	    } else {
		lappend powEditPlotParam(images,new) $name
	    }
	}

    }
    if { $powEditDisplayOpt!="graph" } powEditBuildOptions
}

proc powEditRemoveElems { } {
    global powEditGraphname powEditDisplayOpt
    global powEditPlotParam powEditCurveParam

    set graphNo [.powedit.graphList.lst curselection]
    if { $graphNo=="" } return

    if { $graphNo != "" } {

	set item [.powedit.graphList.lst get $graphNo]
	.powedit.graphList.lst delete $graphNo
	if { [.powedit.graphList.lst size]<=$graphNo } {
	    .powedit.graphList.lst selection set end
	} else {
	    .powedit.graphList.lst selection set $graphNo
	}
	.powedit.elemList.lst insert end $item
	.powedit.elemList.lst selection clear 0 end
	.powedit.elemList.lst selection set end
	foreach {name type} [split $item { }] {}
	if { $type=="(curve)" } {
	    set i [lsearch -exact $powEditPlotParam(curves,new) $name]
	    if { [llength $powEditPlotParam(curves,new)]==1 } {
		set powEditPlotParam(curves,new) NULL
	    } else {
		set powEditPlotParam(curves,new) \
			[lreplace $powEditPlotParam(curves,new) $i $i]
	    }
	} else {
	    set i [lsearch -exact $powEditPlotParam(images,new) $name]
	    if { [llength $powEditPlotParam(images,new)]==1 } {
		set powEditPlotParam(images,new) NULL
	    } else {
		set powEditPlotParam(images,new) \
			[lreplace $powEditPlotParam(images,new) $i $i]
	    }
	}

    }
    if { $powEditDisplayOpt!="graph" } powEditBuildOptions
}

proc powEditUpdateListboxes {  } {
    global powEditGraphname
    global powEditPlotParam powEditCurveParam
    
    .powedit.graphList.lst delete 0 end
    .powedit.elemList.lst  delete 0 end

    set curves ""
    set images ""
    foreach item $powEditPlotParam(curves,new) {
	if {$item!="NULL"} {
	    .powedit.graphList.lst insert end "${item} (curve)"
	    lappend curves $item
	}
    }
    foreach item $powEditPlotParam(images,new) {
	if {$item!="NULL"} {
	    .powedit.graphList.lst insert end "${item} (image)"
	    lappend images $item
	}
    }

    foreach item [powListCurves] {
	if { [lsearch -exact $curves $item]==-1 } {
	    .powedit.elemList.lst insert end "${item} (curve)"
	}
    }
    foreach item [powListImages] {
	if { [lsearch -exact $images $item]==-1 } {
	    .powedit.elemList.lst insert end "${item} (images)"
	}
    }
    if { [.powedit.elemList.lst size]>0 } {
	.powedit.elemList.lst selection set 0
    }
    if { [.powedit.graphList.lst size]>0 } {
	.powedit.graphList.lst selection set 0
    }
}

proc powEditInitVariables { } {
    global powEditGraphname
    global powPlotParam powCurveParam
    global powEditPlotParam powEditCurveParam

    # Start fresh
    catch {unset powEditPlotParam}
    catch {unset powEditCurveParam}

    foreach el [array names powPlotParam]  {
	set p1 [lindex [split $el ,] 0]
	set p2 [lindex [split $el ,] 1]
	if { $p2 == $powEditGraphname } {
	    set powEditPlotParam($p1,old) $powPlotParam($p1,$p2)
	    set powEditPlotParam($p1,new) $powPlotParam($p1,$p2)
        }
    }
    # Hide some plot parameters
    foreach el [list xo yo handletext FixedAspect] {
	unset powEditPlotParam($el,new)
    }

    foreach el [array names powCurveParam]  {
	set p1 [lindex [split $el ,] 0]
	set p2 [lindex [split $el ,] 1]
	if { $p2 == $powEditGraphname } {
	    set powEditCurveParam($p1,old) $powCurveParam($p1,$p2)
	    set powEditCurveParam($p1,new) $powCurveParam($p1,$p2)
        }
    }
    powEditUpdateListboxes
}

proc powEditUpdateVariables { } {
    global powEditGraphname
    global powPlotParam powCurveParam
    global powEditPlotParam powEditCurveParam

    foreach el [array names powEditPlotParam]  {
	set p1 [lindex [split $el ,] 0]
	set p2 [lindex [split $el ,] 1]
	if { $p2 == "new" } {
	    set powPlotParam($p1,$powEditGraphname) $powEditPlotParam($p1,new)
	}
    }

    # Set default values of graph's curves if not already defined

    foreach crv $powEditPlotParam(curves,new) {
	powEditCurveDefOptions $crv
    }

    foreach el [array names powEditCurveParam]  {
	set p1 [lindex [split $el ,] 0]
	set p2 [lindex [split $el ,] 1]
	if { $p2 == "new" } {
	    set powCurveParam($p1,$powEditGraphname) $powEditCurveParam($p1,new)
	}
    }

}

proc powEditGraphOptions { frame } {
    global powbg powEditPlotParam

    if {[winfo exists $frame]} {destroy $frame}
    frame $frame -bg $powbg
    
    set row 1

# Setup Bbox options

    frame $frame.bbox -bg $powbg

    label $frame.bbox.x -bg $powbg -text "X Axis"
    label $frame.bbox.y -bg $powbg -text "Y Axis"
    grid $frame.bbox.x  -in $frame.bbox -row 1 -column 2 -sticky ew
    grid $frame.bbox.y  -in $frame.bbox -row 1 -column 3 -sticky ew

    set prow 2
    foreach {lbl val} {Label: label Min: Bot Max: Top Units: units} {
	label $frame.bbox.l$val -bg $powbg -text $lbl
	entry $frame.bbox.ex$val -bg $powbg \
		-textvariable powEditPlotParam(x$val,new) -takefocus 1
	entry $frame.bbox.ey$val -bg $powbg \
		-textvariable powEditPlotParam(y$val,new) -takefocus 1
	grid $frame.bbox.l$val  -in $frame.bbox -row $prow -column 1 -sticky e \
		-padx 5
	grid $frame.bbox.ex$val -in $frame.bbox -row $prow -column 2 -sticky w \
		-padx 5
	grid $frame.bbox.ey$val -in $frame.bbox -row $prow -column 3 -sticky w \
		-padx 5
	incr prow
    }
    button $frame.bbox.reset -bg $powbg -text "Reset Min/Max" -takefocus 0 \
	    -command {
	foreach par [list xBot yBot xTop yTop] {
	    set powEditPlotParam($par,new) NULL
	}
    }
    grid $frame.bbox.reset -in $frame.bbox -row $prow -column 2 \
	    -columnspan 2 -sticky {} -padx 5 -pady 5

    grid $frame.bbox -in $frame -row $row -column 1 -columnspan 6 \
	    -padx 5 -pady 5
    incr row

    grid rowconfigure $frame $row -minsize 10
    incr row

# Setup Tick Options

    label $frame.nTicks -bg $powbg -text "# of Ticks:"
    grid $frame.nTicks -in $frame -row $row -column 2 -sticky ne

    frame $frame.tickSlide -bg $powbg
    label $frame.tickSlide.few -text "Few" -bg $powbg
    label $frame.tickSlide.lots -text "Many" -bg $powbg
    scale $frame.tickSlide.slide -from 1 -to 12 \
	    -orient horizontal -variable powEditPlotParam(NumTicks,new) \
	    -highlightbackground $powbg -bg $powbg \
	    -showvalue 0 -takefocus 0
    pack $frame.tickSlide.few -in $frame.tickSlide -side left
    pack $frame.tickSlide.slide -in $frame.tickSlide -side left \
	    -expand 1 -fill x
    pack $frame.tickSlide.lots -in $frame.tickSlide -side right
    grid $frame.tickSlide -in $frame -row $row -column 3 -columnspan 4 \
	    -sticky ew -padx 5
    incr row

    grid rowconfigure $frame $row -minsize 10
    incr row

# Setup Grid Options

    checkbutton $frame.grid -bg $powbg -text "Grid Lines -" \
	    -variable powEditPlotParam(Grid,new) \
	    -highlightthickness 0
    grid $frame.grid -in $frame -row $row -column 1 -sticky w -columnspan 6
    incr row

    # Color

    label $frame.gridColor -bg $powbg -text "Color:"
    grid $frame.gridColor -in $frame -row $row -column 2 -sticky ne

    set col 3
    foreach color [list Black Red Yellow White Blue Purple Grey Green Brown] {
	if {$col==6} {set col 3; incr row}
	radiobutton $frame.gridColor$color -bg $powbg -text $color \
		-variable powEditPlotParam(GridColor,new) -value $color \
		-highlightthickness 0 -takefocus 0
	grid $frame.gridColor$color -in $frame \
		-row $row -column $col -sticky nw -padx 4
	incr col
    }
    incr row

    grid rowconfigure $frame $row -minsize 7
    incr row

    # Line Style

    label $frame.style -bg $powbg -text "Style:"
    grid $frame.style -in $frame -row $row -column 2 -sticky ne

    set col 3
    foreach {style val} \
	    {Solid " " "Sm Dash" 10 "Lg Dash" 20 \
	     "Dotted" "4 4" "Dot Dash" "15 10 4 10"} {
	regsub -all { } $style {_} cln
	if {$col==6} {set col 3; incr row}
	radiobutton $frame.lStyle$cln -bg $powbg -text $style \
		-variable powEditPlotParam(GridDash,new) -value $val \
		-highlightthickness 0 -takefocus 0
	grid $frame.lStyle$cln -in $frame \
		-row $row -column $col -sticky nw -padx 4
	incr col
    }
    incr row

}

proc powEditCurveOptions { frame curve } {
    global powbg powEditCurveParam powCurveParam

# If frame already exists delete it

    if {[winfo exists $frame]} {
	destroy $frame
    }

# Set default values if not defined for this curve

    powEditCurveDefOptions $curve

# Create frame and build widgets

    frame $frame -bg $powbg
    set row 1

# Setup Color Choices

    label $frame.color -bg $powbg -text "Color:"
    grid $frame.color -in $frame -row $row -column 1 -sticky ne -columnspan 2

    set col 3
    foreach color [list Black Red Yellow White Blue Purple Grey Green Brown] {
	if {$col==6} {set col 3; incr row}
	radiobutton $frame.lColor$color -bg $powbg -text $color \
		-variable powEditCurveParam(Color${curve},new) -value $color \
		-highlightthickness 0 -takefocus 0
	grid $frame.lColor$color -in $frame \
		-row $row -column $col -sticky nw -padx 4
	incr col
    }
    incr row

    grid rowconfigure $frame $row -minsize 10
    incr row

# Setup Points Options

    checkbutton $frame.pDisp -bg $powbg -text "Points -" \
	    -variable powEditCurveParam(pDisp${curve},new) \
	    -highlightthickness 0
    grid $frame.pDisp -in $frame -row $row -column 1 -sticky w -columnspan 2
    incr row

    grid rowconfigure $frame $row -minsize 7
    incr row

    label $frame.shape -bg $powbg -text "Shape:"
    grid $frame.shape -in $frame -row $row -column 2 -sticky ne

    set col 3
    set cnt 1
    foreach shape {Cross Diamond Box Octagon Triangle "Inv. Triangle"} {
	if {$col==6} {set col 3; incr row}
	radiobutton $frame.pShape$cnt -bg $powbg -text $shape \
		-variable powEditCurveParam(pShape${curve},new) -value $shape \
		-highlightthickness 0 -takefocus 0
	grid $frame.pShape$cnt -in $frame \
		-row $row -column $col -sticky nw -padx 4
	incr col
	incr cnt
    }
    incr row

    grid rowconfigure $frame $row -minsize 7
    incr row

    label $frame.size -bg $powbg -text "Size:"
    grid $frame.size -in $frame -row $row -column 2 -sticky ne

    frame $frame.pSize -bg $powbg
    radiobutton $frame.pSize.fixed -bg $powbg -text "Fixed... " \
	    -variable powEditCurveParam(pSizeErr${curve},new) -value 0 \
	    -highlightthickness 0 -takefocus 0
    label $frame.pSize.text -bg $powbg -width 2 \
	    -textvariable powEditCurveParam(pSize${curve},new)
    label $frame.pSize.pt -bg $powbg -width 2 -text pt
    scale $frame.pSize.slide -from 2 -to 12 -orient horizontal \
	    -variable powEditCurveParam(pSize${curve},new) \
	    -highlightbackground $powbg -bg $powbg \
	    -showvalue 0 -takefocus 0

    grid $frame.pSize.fixed -in $frame.pSize -row 1 -column 1 -sticky nw -padx 3
    grid $frame.pSize.text -in $frame.pSize -row 1 -column 2 -sticky ne
    grid $frame.pSize.pt -in $frame.pSize -row 1 -column 3 -sticky nw
    grid $frame.pSize.slide -in $frame.pSize -row 1 -column 4 -sticky new \
	    -padx 4
    
    grid columnconfigure $frame.pSize 4 -weight 1
    grid $frame.pSize -in $frame -row $row -column 3 -columnspan 3 \
	    -sticky new

    incr row

    radiobutton $frame.pSizeError -bg $powbg -text Errorbars \
	    -variable powEditCurveParam(pSizeErr${curve},new) -value 1 \
	    -highlightthickness 0 -takefocus 0
    grid $frame.pSizeError -in $frame \
	    -row $row -column 3 -sticky nw -padx 4
    incr row

    grid rowconfigure $frame $row -minsize 7
    incr row

    label $frame.fill -bg $powbg -text "Fill:"
    grid $frame.fill -in $frame -row $row -column 2 -sticky ne

    set col 3
    foreach {lab val} {Yes Yes No No} {
	if {$col==7} {set col 3; incr row}
	radiobutton $frame.pFill$val -bg $powbg -text $lab \
		-variable powEditCurveParam(pFill${curve},new) -value $val \
		-highlightthickness 0 -takefocus 0
	grid $frame.pFill$val -in $frame \
		-row $row -column $col -sticky nw -padx 4
	incr col
    }
    incr row

# Setup Lines Options

    grid rowconfigure $frame $row -minsize 10
    incr row

    checkbutton $frame.lDisp -bg $powbg -text "Lines -" \
	    -variable powEditCurveParam(lDisp${curve},new) \
	    -highlightthickness 0 
    grid $frame.lDisp -in $frame -row $row -column 1 -sticky w -columnspan 2
    incr row

    label $frame.style -bg $powbg -text "Style:"
    grid $frame.style -in $frame -row $row -column 2 -sticky ne

    set col 3
    foreach {style val} \
	    {Solid " " "Sm Dash" 10 "Lg Dash" 20 \
	     "Dotted" "4 4" "Dot Dash" "15 10 4 10"} {
	regsub -all { } $style {_} cln
	if {$col==6} {set col 3; incr row}
	radiobutton $frame.lStyle$cln -bg $powbg -text $style \
		-variable powEditCurveParam(lStyle${curve},new) -value $val \
		-highlightthickness 0 -takefocus 0
	grid $frame.lStyle$cln -in $frame \
		-row $row -column $col -sticky nw -padx 4
	incr col
    }
    incr row

    grid rowconfigure $frame $row -minsize 7
    incr row

    label $frame.width -bg $powbg -text "Width:"
    grid $frame.width -in $frame -row $row -column 2 -sticky ne

    set col 3
    foreach {size val} {Thin 1 Medium 2 Thick 3} {
	if {$col==7} {set col 3; incr row}
	radiobutton $frame.lWidth$size -bg $powbg -text $size \
		-variable powEditCurveParam(lWidth${curve},new) -value $val \
		-highlightthickness 0 -takefocus 0
	grid $frame.lWidth$size -in $frame \
		-row $row -column $col -sticky nw -padx 4
	incr col
    }
    incr row

    grid rowconfigure $frame $row -minsize 7
    incr row

    label $frame.step -bg $powbg -text "Point Connect:"
    grid $frame.step -in $frame -row $row -column 2 -sticky ne

    set col 3
    foreach {style val} {"Normal" 0 "Histogram" 1} {
	if {$col==7} {set col 3; incr row}
	radiobutton $frame.lStep$val -bg $powbg -text $style \
		-variable powEditCurveParam(lStep${curve},new) -value $val \
		-highlightthickness 0 -takefocus 0
	grid $frame.lStep$val -in $frame \
		-row $row -column $col -sticky nw -padx 4
	incr col
    }
    incr row

    grid columnconfigure $frame 1 -minsize 10
}

proc powEditCurveDefOptions { curve } {
    global powEditCurveParam powCurveParam powPlotParam

    if { $curve == "NULL" || $curve == "" } return

    set crvGraph ""
    foreach opt $powCurveParam(allOpts,powDef) {
	if {! [info exists powEditCurveParam(${opt}${curve},new)]} {
	    if { $crvGraph=="" } {
		set crvGraph powDef
		foreach grph [powListGraphs] {
		    if { [regexp "scope$" $grph] } continue
		    if { [lsearch -exact $powPlotParam(curves,$grph) $curve] \
			    != -1 } {
			set crvGraph $grph
		    }
		}
	    }
	    if { $crvGraph=="powDef" } {
		set powEditCurveParam(${opt}${curve},new) \
			$powCurveParam(${opt},powDef)
		set powEditCurveParam(${opt}${curve},old) \
			$powCurveParam(${opt},powDef)
	    } else {
		set powEditCurveParam(${opt}${curve},new) \
			$powCurveParam(${opt}${curve},$crvGraph)
		set powEditCurveParam(${opt}${curve},old) \
			$powCurveParam(${opt}${curve},$crvGraph)
	    }
	}
    }
}

###########################################################################
#                                                                         #
#               Routines for editting Objects                             #
#                                                                         #
###########################################################################

proc powEditObjectDlg {  } {
    global powPlotParam powCurveParam powbg
    global powEditObject

    if {[winfo exists .powobject]} {destroy .powobject}
    powToplevel .powobject .pow "-bg $powbg"
    catch {wm title .powobject "Edit Objects"}

    label .powobject.objectCont -text "All Objects:" -fg black -bg yellow
    frame .powobject.objectList -bg $powbg
    scrollbar .powobject.objectList.scrolly -orient vertical -takefocus 0 \
	    -command {.powobject.objectList.lst yview} -bg $powbg
    listbox .powobject.objectList.lst -bg $powbg -width 25 -height 6 \
	    -selectmode browse -exportselection 0 -takefocus 0 \
	    -yscrollcommand {.powobject.objectList.scrolly set }
    bind .powobject.objectList.lst <ButtonRelease-1> \
	    { powEditUpdateObject }

    grid .powobject.objectList.lst     -in .powobject.objectList \
	    -row 1 -column 1 -sticky news
    grid .powobject.objectList.scrolly -in .powobject.objectList \
	    -row 1 -column 2 -sticky news
    grid rowconfigure    .powobject.objectList 1 -weight 1
    grid columnconfigure .powobject.objectList 1 -weight 1

    label .powobject.dataCont -text "All Data:" -fg black -bg yellow
    frame .powobject.dataList -bg $powbg
    scrollbar .powobject.dataList.scrolly -orient vertical -takefocus 0 \
	    -command {.powobject.dataList.lst yview} -bg $powbg
    listbox .powobject.dataList.lst -bg $powbg -width 25 -height 6 \
	    -selectmode browse -exportselection 0 -takefocus 0 \
	    -yscrollcommand {.powobject.dataList.scrolly set }

    grid .powobject.dataList.lst     -in .powobject.dataList -row 1 -column 1 \
	    -sticky news
    grid .powobject.dataList.scrolly -in .powobject.dataList -row 1 -column 2 \
	    -sticky news
    grid rowconfigure    .powobject.dataList 1 -weight 1
    grid columnconfigure .powobject.dataList 1 -weight 1

    button .powobject.help -bg $powbg -text "Help" \
	    -command {powHelp EditObjects.html} -takefocus 0

    grid .powobject.objectCont -in .powobject -row 1 -column 1 -sticky w \
	    -padx 5 -pady 5
    grid .powobject.objectList -in .powobject -row 2 -column 1 -sticky news
    grid .powobject.help       -in .powobject -row 1 -column 2
    grid .powobject.dataCont   -in .powobject -row 1 -column 3 -sticky w \
	    -padx 5 -pady 5
    grid .powobject.dataList   -in .powobject -row 2 -column 3 -sticky news

    button .powobject.editData -text "Edit Data" -bg $powbg -takefocus 0 \
	    -command {
	if { [info exists powDontPush] } {
	    puts "I told you not to push that button again."
	} else {
	    puts "Don't push that button again."
	    set powDontPush 1
	}
    }
    grid .powobject.editData -i .powobject -row 3 -column 3 -sticky n -pady 2

    frame .powobject.objectName -bg $powbg
    label .powobject.objectName.lab -bg yellow -fg black -text "Object Name:"
    entry .powobject.objectName.ent -textvariable powEditObject(name) \
	    -bg $powbg

    pack .powobject.objectName.lab -in .powobject.objectName -side left
    pack .powobject.objectName.ent -in .powobject.objectName -side left -padx 5
    grid .powobject.objectName     -in .powobject -row 3 -column 1 \
	    -columnspan 2 -sticky w -padx 18

    frame .powobject.objectType -bg $powbg -relief ridge -bd 3
    label .powobject.objectType.lab -bg yellow -fg black -text "Object Type:"
    radiobutton .powobject.objectType.curve -bg $powbg -text Curve \
	    -variable powEditObject(type) -value "(curve)" \
	    -highlightthickness 0 -takefocus 0
    radiobutton .powobject.objectType.image -bg $powbg -text Image \
	    -variable powEditObject(type) -value "(image)" \
	    -highlightthickness 0 -takefocus 0
    set powEditObject(type) "(curve)"

    pack .powobject.objectType.lab -in .powobject.objectType -side left
    pack .powobject.objectType.curve -in .powobject.objectType \
	    -side left -padx 10
    pack .powobject.objectType.image -in .powobject.objectType \
	    -side left -padx 10
    grid .powobject.objectType -in .powobject -row 4 -column 1 -sticky sew \
	    -columnspan 3 -padx 15

    frame .powobject.btns -bg $powbg
    button .powobject.btns.create -text "Create Object" -bg $powbg \
	    -command {powEditCreateObject}
    button .powobject.btns.reload -text "Reload Info" -bg $powbg \
	    -command {powEditLoadObjects; powEditBuildObject}
    button .powobject.btns.exit -text "Exit"  -bg $powbg \
	    -command {destroy .powobject}

    pack .powobject.btns.create -in .powobject.btns -side left \
	    -padx 5 -pady 2
    pack .powobject.btns.reload -in .powobject.btns -side left \
	    -padx 5 -pady 2
    pack .powobject.btns.exit   -in .powobject.btns -side left \
	    -padx 5 -pady 2
    grid .powobject.btns        -in .powobject -row 6 -column 1 -columnspan 3 \
	    -pady 5

    grid rowconfigure    .powobject 2 -weight 1
    grid rowconfigure    .powobject 5 -weight 1

    grid columnconfigure .powobject 1 -weight 1
    grid columnconfigure .powobject 3 -weight 1

    powEditLoadObjects
    powEditBuildObject
    trace variable powEditObject(type) w powChangeBuildObject
}

proc powEditLoadObjects { {init 1} } {
    
    .powobject.objectList.lst delete 0 end
    .powobject.dataList.lst delete 0 end

    set last "NULL (curve)"
    foreach itm [powListCurves] {
	set last "$itm (curve)"
	.powobject.objectList.lst insert end $last
    }

    foreach itm [powListImages] {
	set last "$itm (image)"
	.powobject.objectList.lst insert end $last
    }
    
    foreach itm [powListData] {
	.powobject.dataList.lst insert end "$itm ([powFetchDataLength $itm])"
    }
    .powobject.dataList.lst insert end "NULL (0)"
    .powobject.dataList.lst selection set 0

    if {$init} {
	.powobject.objectList.lst selection set end
	eval [concat powEditInitObject $last]
    }
}

proc powEditInitObject { obj typ } {
    global powEditObject powWCS

    if {$typ == "(curve)"} {
	set loc [lsearch -exact [powListCurves] $obj]
    } else {
	set loc [lsearch -exact [powListImages] $obj]
    }

    set powEditObject(name) $obj
    set powEditObject(type) $typ

    # Initialize all the entries to standard default NULLS or values

    foreach el [list xdata xedata xunits ydata yedata yunits \
	             zdata zedata zunits] {
        set powEditObject($el) NULL
    }
    foreach el [list xdim ydim xorigin yorigin xinc yinc] {
        set powEditObject($el) 1
    }

    set powEditObject(wcs) 0
    foreach el [list xref yref xrefpix yrefpix xinc yinc rot ctype] {
	set powEditObject(wcs$el) " "
    }

    # Now fill in the known quantities

    if { $obj!="NULL" && $loc!=-1 } {

	if { $typ=="(image)" } {
	    array set powEditObject [powFetchImageInfoHash $obj]
	    set powEditObject(xdata) $powEditObject(data)
	    set powEditObject(xdim) $powEditObject(width)
	    set powEditObject(ydim) $powEditObject(height)
	} else {
	    array set powEditObject [powFetchCurveInfoHash $obj]
	    foreach {vec lc} {X x Y y XE xe YE ye} {
		if { $powEditObject($vec) != "NULL" } {
		    array set tmpArray \
			    [powFetchVectorInfoHash $powEditObject($vec)]
		    set powEditObject(${lc}data) $tmpArray(data)
		    set powEditObject(${lc}units) $tmpArray(units)
		} else {
		    set powEditObject(${lc}data) NULL
		    set powEditObject(${lc}units) NULL
		}
	    }
	}

	if {[info exists powWCS($obj)] && $powWCS($obj) != "" } {
	    set powEditObject(wcs) 1
	    set wcslist $powWCS($obj)
	    set wcscnt 0
	    foreach wcspar \
		[list xref yref xrefpix yrefpix xinc yinc rot ctype] {
		    set powEditObject(wcs$wcspar) \
			[lindex $wcslist $wcscnt]
		    incr wcscnt
		}
	}
    }
}

proc powEditCreateObject { } {
    global powEditObject powWCS

    set obj $powEditObject(name)
    set wcslist ""
    if { $powEditObject(wcs) } {
	foreach par [list xref yref xrefpix yrefpix xinc yinc rot ctype] {
	    lappend wcslist $powEditObject(wcs$par)
	}
    }
    set powWCS($obj) $wcslist

    if { $powEditObject(type)=="(curve)" } {

	set vectors ""
	foreach vec { x xe y ye } {
	    if { $powEditObject(${vec}data) == "NULL" } {
		lappend vectors NULL
	    } else {
		powCreateVector ${obj}_${vec}data $powEditObject(${vec}data) 0 \
			NULL $powEditObject(${vec}units)
		lappend vectors ${obj}_${vec}data
	    }
	}
	eval [concat powCreateCurve $obj $vectors]

    } else {

	if { $powEditObject(wcs) } {
	    powCreateImage $obj $powEditObject(xdata) \
		    0 0 \
		    $powEditObject(xdim) $powEditObject(ydim) \
		    1 1 1 1 \
		    $powEditObject(xunits) $powEditObject(yunits) \
		    $powEditObject(zunits)
	} else {
	    powCreateImage $obj $powEditObject(xdata) \
		    0 0 \
		    $powEditObject(xdim) $powEditObject(ydim) \
		    $powEditObject(xorigin) $powEditObject(xinc) \
		    $powEditObject(yorigin) $powEditObject(yinc) \
		    $powEditObject(xunits) $powEditObject(yunits) \
		    $powEditObject(zunits)
	}

    }
    powEditLoadObjects 0
    set elem "$obj $powEditObject(type)"
    set num [lsearch -exact [.powobject.objectList.lst get 0 end] $elem]
    .powobject.objectList.lst selection set $num
    eval [concat powEditInitObject $elem]
}

proc powEditUpdateObject { } {
    set itm [.powobject.objectList.lst curselection]
    if {$itm==""} {
	powEditInitObject NULL "(curve)"
    } else {
	eval [concat powEditInitObject [.powobject.objectList.lst get $itm]]
    }
    powEditBuildObject
}

proc powChangeBuildObject { a b c } {
    powEditBuildObject
}

proc powEditBuildObject { } {
    global powEditObject powbg
    
    if { $powEditObject(type)=="(curve)" } {
	powEditBuildCurve .powobject.curveFrame
	.powobject.curveFrame configure -borderwidth 3 -relief ridge
	grid .powobject.curveFrame -in .powobject -row 5 -column 1 \
		-padx 15 -pady 5 -columnspan 3 -sticky news
	catch {grid remove .powobject.imageFrame}
    } else {
	powEditBuildImage .powobject.imageFrame
	.powobject.imageFrame configure -borderwidth 3 -relief ridge
	grid .powobject.imageFrame -in .powobject -row 5 -column 1 \
		-padx 15 -pady 5 -columnspan 3 -sticky news
	catch {grid remove .powobject.curveFrame}
    }
}

proc powEditUpdateDataButton { btn var } {
    global powEditObject

    set elem [.powobject.dataList.lst curselection]
    if { $elem=="" } return

    set data [lindex [.powobject.dataList.lst get $elem] 0]
    set powEditObject($var) $data
    $btn configure -text $data
}

proc powEditBuildCurve { frame } {
    global powEditObject powbg

    if { [winfo exists $frame] } {
	foreach {lbl var} [list Data: data Error: edata] {
	    $frame.bx$var configure -text $powEditObject(x$var)
	    $frame.by$var configure -text $powEditObject(y$var)
	}
	return
    }
    frame $frame -bg $powbg

    set row 1
    grid columnconfigure $frame 1 -minsize 10
    grid columnconfigure $frame 4 -minsize 10

    label $frame.x -bg $powbg -text X
    label $frame.y -bg $powbg -text Y
    grid $frame.x -in $frame -row $row -column 3 -sticky s
    grid $frame.y -in $frame -row $row -column 5 -sticky s
    incr row

    # Create data buttons

    foreach {lbl var} [list Data: data Error: edata] {
	label $frame.l$var -bg $powbg -text $lbl
	grid $frame.l$var -in $frame -row $row -column 2 -sticky e -padx 10
	button $frame.bx$var -bg $powbg -text $powEditObject(x$var) \
		-command "powEditUpdateDataButton $frame.bx$var x$var"
	button $frame.by$var -bg $powbg -text $powEditObject(y$var) \
		-command "powEditUpdateDataButton $frame.by$var y$var"
	grid $frame.bx$var -in $frame -row $row -column 3 -sticky news
	grid $frame.by$var -in $frame -row $row -column 5 -sticky news
	incr row
    }

    foreach {lbl var} [list Units: units] {
	label $frame.l$var -bg $powbg -text $lbl
	grid $frame.l$var -in $frame -row $row -column 2 -sticky e -padx 10
	entry $frame.ex$var -bg $powbg -textvariable powEditObject(x$var) \
		 -takefocus 1
	entry $frame.ey$var -bg $powbg -textvariable powEditObject(y$var) \
		 -takefocus 1
	grid $frame.ex$var -in $frame -row $row -column 3 -sticky news
	grid $frame.ey$var -in $frame -row $row -column 5 -sticky news
	incr row
    }

    grid rowconfigure $frame $row -minsize 10
    incr row

    powEditBuildWCS $frame $row
}

proc powEditBuildImage { frame } {
    global powEditObject powbg

    if { [winfo exists $frame] } {
	$frame.bdata configure -text $powEditObject(xdata)
	return
    }
    frame $frame -bg $powbg

    set row 1
    grid columnconfigure $frame 1 -minsize 10
    grid columnconfigure $frame 4 -minsize 10

    label $frame.ldata -bg $powbg -text "Data:"
    grid $frame.ldata -in $frame -row $row -column 2 -sticky e -padx 10
    button $frame.bdata -bg $powbg -text $powEditObject(xdata) \
	    -command "powEditUpdateDataButton $frame.bdata xdata;
	              set powEditObject(ydim) 1;
                      set powEditObject(xdim) \
                            \[powFetchDataLength \$powEditObject(xdata)\]"
    grid $frame.bdata -in $frame -row $row -column 3 -sticky w -columnspan 3
    incr row

    label $frame.lzunits -bg $powbg -text "Units:"
    grid $frame.lzunits -in $frame -row $row -column 2 -sticky e -padx 10
    entry $frame.ezunits -bg $powbg -textvariable powEditObject(zunits) \
	     -takefocus 1
    grid $frame.ezunits -in $frame -row $row -column 3 -sticky w
    incr row

    grid rowconfigure $frame $row -minsize 10
    incr row

    label $frame.x -bg $powbg -text X
    label $frame.y -bg $powbg -text Y
    grid $frame.x -in $frame -row $row -column 3 -sticky s
    grid $frame.y -in $frame -row $row -column 5 -sticky s
    incr row

    foreach {lbl var} \
	    [list Dimensions: dim Origin: origin "Pixel Size:" inc Units: units] {
	label $frame.l$var -bg $powbg -text $lbl
	grid $frame.l$var -in $frame -row $row -column 2 -sticky e -padx 10
	entry $frame.ex$var -bg $powbg -textvariable powEditObject(x$var) \
		 -takefocus 1
	entry $frame.ey$var -bg $powbg -textvariable powEditObject(y$var) \
		 -takefocus 1
	grid $frame.ex$var -in $frame -row $row -column 3 -sticky news
	grid $frame.ey$var -in $frame -row $row -column 5 -sticky news
	incr row
    }

    grid rowconfigure $frame $row -minsize 10
    incr row

    powEditBuildWCS $frame $row
}

proc powEditBuildWCS { frame row } {
    global powEditObject powbg

# Build WCS entries

    checkbutton $frame.wcs -bg $powbg -text "WCS Info -" \
	    -highlightthickness 0 -takefocus 0 \
	    -variable powEditObject(wcs)
    grid $frame.wcs -in $frame -row $row -column 1 -columnspan 2 -sticky w
    incr row
			      
    foreach {lbl var} \
	    [list "Ref Value:" ref "Ref Pixel:" refpix "Pixel Scale:" inc] {
	label $frame.lw$var -bg $powbg -text $lbl
	grid $frame.lw$var -in $frame -row $row -column 2 -sticky e -padx 10
	entry $frame.ewx$var -bg $powbg -textvariable powEditObject(wcsx$var) \
		 -takefocus 1
	entry $frame.ewy$var -bg $powbg -textvariable powEditObject(wcsy$var) \
		 -takefocus 1
	grid $frame.ewx$var -in $frame -row $row -column 3 -sticky news
	grid $frame.ewy$var -in $frame -row $row -column 5 -sticky news
	incr row
    }

    label $frame.lrot -bg $powbg -text "Rotation:"
    grid $frame.lrot -in $frame -row $row -column 2 -sticky e -padx 10
    entry $frame.erot -bg $powbg -textvariable powEditObject(wcsrot) \
	     -takefocus 1
    grid $frame.erot -in $frame -row $row -column 3 -sticky news
    incr row

    label $frame.lproj -bg $powbg -text "Projection:"
    grid $frame.lproj -in $frame -row $row -column 2 -sticky ne -padx 10

    set pcol 1
    set prow 1
    frame $frame.projType -bg $powbg
    foreach {lbl val} \
	    {TAN "-TAN" SIN "-SIN" ARC "-ARC" NCP "-NCP" \
             GLS "-GLS" MER "-MER" AIT "-AIT" STG "-STG"} {
	 if {$pcol==5} {set pcol 1; incr prow}
	 radiobutton $frame.projType.p$lbl -bg $powbg -text $lbl \
		 -variable powEditObject(wcsctype) -value $val \
		 -highlightthickness 0 -takefocus 0
	 grid $frame.projType.p$lbl -in $frame.projType \
		 -row $prow -column $pcol -sticky nw -padx 6
	 incr pcol
    }

    grid $frame.projType -in $frame -row $row -column 3 -columnspan 3 -sticky w
    incr row
}

########################################################################
#                                                                      #
#              Functions to set Default Curve Options                  #
#                                                                      #
########################################################################

proc powCurveOptions { } {
    global currgn powbg powCurveParam powEditCurveParam

    if {[winfo exists .powcurve]} {destroy .powcurve}
    powToplevel .powcurve .pow "-bg $powbg"
    catch {wm title .powcurve "Default Curve Options"}
    
    foreach opt $powCurveParam(allOpts,powDef) {
	set powEditCurveParam(${opt},new) $powCurveParam(${opt},powDef)
    }

    powEditCurveOptions .powcurve.opts ""
    .powcurve.opts configure -relief ridge -bd 3

# Setup Buttons

    button .powcurve.help -text "Help" \
	    -command {powHelp CurveOptions.html} \
	    -bg $powbg -takefocus 0
    
    frame .powcurve.buttons -bg $powbg
    button .powcurve.buttons.reset -text "Reset" -bg $powbg \
	    -command {
	foreach opt $powCurveParam(allOpts,powDef) {
	    set powEditCurveParam(${opt},new) $powCurveParam(${opt},powDef)
	}
    }
    button .powcurve.buttons.exit -text "Exit" -bg $powbg \
	    -command {
	foreach opt $powCurveParam(allOpts,powDef) {
	    set powCurveParam(${opt},powDef) $powEditCurveParam(${opt},new)
	}
	destroy .powcurve
    }
    pack .powcurve.buttons.reset -in .powcurve.buttons -side left \
	    -padx 4 -pady 3
    pack .powcurve.buttons.exit -in .powcurve.buttons -side left \
	    -padx 4 -pady 3

    label .powcurve.lab -bg $powbg -text "Default curve options for new curves:"

# Place everything into dialog

    grid .powcurve.help -in .powcurve -row 1 -column 1 -sticky ne
    grid .powcurve.lab -in .powcurve -row 2 -column 1
    grid .powcurve.opts -in .powcurve -row 3 -column 1 \
	    -sticky news -padx 15 -pady 10
    grid .powcurve.buttons -in .powcurve -row 4 -column 1 -pady 8
}

